import { ForbiddenException, Injectable } from "@nestjs/common";
import { PrismaService } from "src/prisma/prisma.service";
import { AuthDto } from "./dto";
import * as argon from "argon2";
import { PrismaClientKnownRequestError } from "@prisma/client/runtime/library";
import { JwtService } from "@nestjs/jwt";
import { User } from "@prisma/client";
import { time } from "console";

@Injectable({})
export class AuthService {
  constructor(
    private prisma: PrismaService,
    private jwt: JwtService,
  ) {}

  async login(dto: AuthDto) {
    const User = await this.prisma.user.findUnique({
      where: {
        email: dto.email,
      },
    });
    if (!User) {
      console.warn(`ACCESS: Refused login for "${dto.email}" (email not used)`);
      throw new ForbiddenException("Credential(s) invalid.");
    }

    const pwMatches: boolean = await argon.verify(User.hash, dto.password);
    if (!pwMatches) {
      console.warn(
        `ACCESS: Refused login for "${dto.email}" (invalid password)`,
      );
      throw new ForbiddenException("Credential(s) invalid.");
    }

    delete User.hash;
    console.info(`ACCESS: Granted login for "${dto.email}"`);
    return User;
  }

  async register(dto: AuthDto) {
    const userPasswordHash = await argon.hash(dto.password);
    try {
      const User = await this.prisma.user.create({
        data: {
          email: dto.email,
          hash: userPasswordHash,
        },
        select: {
          id: true,
          email: true,
          firstName: true,
          lastName: true,
        },
      });
      //delete User.hash;
      return User;
    } catch (error) {
      if (error instanceof PrismaClientKnownRequestError) {
        if (error.code === "P2002") {
          throw new ForbiddenException("Credential(s) taken.");
        }
      }
    }
  }

  private async generateAuthToken(targetUser: User, dayToLive: number) {
    const timestamp = Date.now();

    const jwtPayload = {
      sub: targetUser.id,
      iat: timestamp,
    };

    return this.jwt.signAsync(jwtPayload, {
      expiresIn: `${dayToLive}d`,
      
    })
  }
}
